#========================================================================================
# package      DE_EPAGES::EasyMarketing::t::unit::SnippetManager
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  unit test for DE_EPAGES::EasyMarketing::t::unit::SnippetManager
#========================================================================================
package DE_EPAGES::EasyMarketing::t::unit::SnippetManager;
use base DE_EPAGES::Core::API::Test;

use Test::More;
use Test::Exception;
use DE_EPAGES::Test::Mock::Mockify;
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::EasyMarketing::API::SnippetManager;
use DE_EPAGES::Test::API::TestTools qw ( TestExistsMethod );

use strict;

sub test {
    my $self = shift;
    $self->test_getSnippets_BasketMultiCheckout();
    $self->test_getSnippets_BasketClassicCheckout();
    $self->test_getSnippets_BasketMobileCheckout();
    $self->test_getSnippets_BasketNotFirstPageINCheckout();

    $self->test_getSnippets_CustomerOrder();
    $self->test_getSnippets_Category();
    $self->test_getSnippets_Product();
    $self->test_getSnippets_Shop();

    $self->test_getSnippets_NotExpected();
    $self->test_getSnippets_SystemObject();

    done_testing();
}



sub test_getSnippets_BasketNotFirstPageINCheckout {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ViewAction = 'Wrong View Action';
    my $Basket = $self->_createBasket();
    my $SnippetManager = $self->_createSnippetManager();

    is($SnippetManager->getSnippets($Basket, $ViewAction), 'Snippet for Remarketing BasketPage', "$SubTestName - Test if empty string on the not covered ViewAction");

    my $InputLeadTracker = $SnippetManager->{'LeadTracker'}->{'getSnippet_Params'};
    is_deeply($InputLeadTracker, undef, "$SubTestName - Test if the Basket was not handle over to the LeadTracker object");

    my $InputRemarketing = $SnippetManager->{'RemarketingBasketPage'}->{'getSnippet_Params'};
    is_deeply($InputRemarketing, $Basket, "$SubTestName - Test if the Basket was given to the Remarketing BasketPage object");
}



sub test_getSnippets_BasketMobileCheckout {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ViewAction = 'View';
    my $Basket = $self->_createBasket();
    my $SnippetManager = $self->_createSnippetManager();

    is($SnippetManager->getSnippets($Basket, $ViewAction), 'Snippet for Remarketing BasketPage Snippet for LeadTracker', "$SubTestName - Test if Snippets were return to the basket page on mobile checkout");

    my $InputLeadTracker = $SnippetManager->{'LeadTracker'}->{'getSnippet_Params'};
    is_deeply($InputLeadTracker, $Basket, "$SubTestName - Test if the Basket was not handle over to the LeadTracker object");

    my $InputRemarketing = $SnippetManager->{'RemarketingBasketPage'}->{'getSnippet_Params'};
    is_deeply($InputRemarketing, $Basket, "$SubTestName - Test if the Basket was given to the Remarketing BasketPage object");
}



sub test_getSnippets_BasketClassicCheckout {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ViewAction = 'View';
    my $Basket = $self->_createBasket();
    my $SnippetManager = $self->_createSnippetManager();

    is($SnippetManager->getSnippets($Basket, $ViewAction), 'Snippet for Remarketing BasketPage Snippet for LeadTracker', "$SubTestName - Test if Snippets were return to the basket page on classical checkout");

    my $InputLeadTracker = $SnippetManager->{'LeadTracker'}->{'getSnippet_Params'};
    is_deeply($InputLeadTracker, $Basket, "$SubTestName - Test if the Basket was not handle over to the LeadTracker object");

    my $InputRemarketing = $SnippetManager->{'RemarketingBasketPage'}->{'getSnippet_Params'};
    is_deeply($InputRemarketing, $Basket, "$SubTestName - Test if the Basket was given to the Remarketing BasketPage object");
}



sub test_getSnippets_BasketMultiCheckout {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ViewAction = 'ViewMultiCheckoutBasket';
    my $Basket = $self->_createBasket();
    my $SnippetManager = $self->_createSnippetManager();

    is($SnippetManager->getSnippets($Basket, $ViewAction), 'Snippet for Remarketing BasketPage Snippet for LeadTracker', "$SubTestName - Test if Snippets were return to the basket page on multistep checkout");

    my $InputLeadTracker = $SnippetManager->{'LeadTracker'}->{'getSnippet_Params'};
    is_deeply($InputLeadTracker, $Basket, "$SubTestName - Test if the Basket was not handle over to the LeadTracker object");

    my $InputRemarketing = $SnippetManager->{'RemarketingBasketPage'}->{'getSnippet_Params'};
    is_deeply($InputRemarketing, $Basket, "$SubTestName - Test if the Basket was given to the Remarketing BasketPage object");
}



sub test_getSnippets_NotExpected {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Object =  $self->_createObject();
    my $SnippetManager = $self->_createSnippetManager();

    is($SnippetManager->getSnippets($Object), '', "$SubTestName - Test if no snippet was returned for not expected object");
}



sub test_getSnippets_SystemObject {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $System =  $self->_createSystemObject();
    my $SnippetManager = $self->_createSnippetManager();

    is($SnippetManager->getSnippets($System), '', "$SubTestName - Test if no snippet was returned for system object");
}



sub test_getSnippets_CustomerOrder {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Order = $self->_createCustomerOrder();
    my $SnippetManager = $self->_createSnippetManager();

    is($SnippetManager->getSnippets($Order), 'Snippet for ConversionTracker Snippet for Remarketing ConfirmationPage', "$SubTestName - Test if Snippets were return to the confirmation page");

    my $InputConversionTracker = $SnippetManager->{'ConversionTracker'}->{'getSnippet_Params'};
    is_deeply($InputConversionTracker, $Order, "$SubTestName - Test if the Order was given to the ConversionTracker Object");

    my $InputRemarketing = $SnippetManager->{'RemarketingConfirmationPage'}->{'getSnippet_Params'};
    is_deeply($InputRemarketing, $Order, "$SubTestName - Test if the Order was given to the Remarketing ConfirmationPage Object");
}



sub test_getSnippets_Category {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Category = $self->_createCategory();
    my $SnippetManager = $self->_createSnippetManager();

    is($SnippetManager->getSnippets($Category), 'Snippet for Remarketing CategoryPage', "$SubTestName - Test if Snippets were return to the category page");

    my $InputRemarketing = $SnippetManager->{'RemarketingCategoryPage'}->{'getSnippet_Params'};
    is_deeply($InputRemarketing, $Category, "$SubTestName - Test if the Product was given to the Remarketing CategoryPage Object");
}



sub test_getSnippets_Product {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Product = $self->_createProduct();
    my $SnippetManager = $self->_createSnippetManager();

    is($SnippetManager->getSnippets($Product), 'Snippet for Remarketing ProductPage', "$SubTestName - Test if Snippets were return to the product page");

    my $InputRemarketing = $SnippetManager->{'RemarketingProductPage'}->{'getSnippet_Params'};
    is_deeply($InputRemarketing, $Product, "$SubTestName - Test if the Product was given to the Remarketing ProductPage Object");
}



sub test_getSnippets_Shop {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->_createShop();
    my $SnippetManager = $self->_createSnippetManager();

    is($SnippetManager->getSnippets($Shop), 'Snippet for Remarketing HomePage', "$SubTestName - Test if Snippets were return to the home page");

    my $InputRemarketing = $SnippetManager->{'RemarketingHomePage'}->{'getSnippet_Params'};
    is_deeply($InputRemarketing, $Shop, "$SubTestName - Test if the Product was given to the Remarketing HomePage Object");
}



#---------------------------------------------------------------------------
#  Helpers
#---------------------------------------------------------------------------
sub _createObject{
    my $self = shift;
    my $hAttributes = {
        'instanceOf' => 'Object',
        'Shop' => $self->_createShop(),
        'Site' => $self->_createShop(),
    };
    my $MockedObject = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Object::API::Object::Object', $hAttributes);
    return $MockedObject;
}



sub _createShop{
    my $self = shift;
    my $hAttributes = {
        'instanceOf' => 'Shop',
        'EasyMarketingAccessToken' => 'AccessToken',
    };
    my $MockedShop = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', $hAttributes);
    $MockedShop->set({
        'Shop' => $MockedShop,
        'Site' => $MockedShop,
    });
    TestExistsMethod('DE_EPAGES::Shop::API::Object::Shop', 'class');
    my $HasEasyMarketingAccessToken = 1;
    $MockedShop->mock('class', sub { return $self->_createClass($HasEasyMarketingAccessToken)});
    return $MockedShop;
}



sub _createBasket {
    my $self = shift;
    my $hAttributes = {
        'instanceOf' => 'Basket',
        'Shop' => $self->_createShop(),
        'Site' => $self->_createShop(),
    };
    my $MockedBasket = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Order::API::Object::Basket', $hAttributes);
    return $MockedBasket;
}



sub _createCustomerOrder {
    my $self = shift;
    my $hAttributes = {
        'instanceOf' => 'CustomerOrder',
        'Shop' => $self->_createShop(),
        'Site' => $self->_createShop(),
    };
    my $MockedOrder = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Order::API::Object::CustomerOrder', $hAttributes);
    return $MockedOrder;
}



sub _createProduct{
    my $self = shift;
    my $hAttributes = {
        'instanceOf' => 'Product',
        'Shop' => $self->_createShop(),
        'Site' => $self->_createShop(),
    };
    my $MockedProduct = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Product::API::Object::Product', $hAttributes);
    return $MockedProduct;
}



sub _createCategory{
    my $self = shift;
    my $hAttributes = {
        'instanceOf' => 'Category',
        'Site' => $self->_createShop(),
    };
    my $MockedCategory = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Content::API::Object::Category', $hAttributes);
    return $MockedCategory;
}



sub _createSystemObject{
    my $self = shift;
    my $hAttributes = {
        'instanceOf' => 'Object',
    };
    my $MockedSystem = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Object::API::Object::Site', $hAttributes);
    $MockedSystem->set({
        'Site' => $MockedSystem,
    });
    TestExistsMethod('DE_EPAGES::Object::API::Object::Site', 'class');
    my $HasEasyMarketingAccessToken = 0;
    $MockedSystem->mock('class', sub { return $self->_createClass($HasEasyMarketingAccessToken)});
    return $MockedSystem;
}



sub _createSnippetManager {
    my $self = shift;

    my $SnippetManager = DE_EPAGES::EasyMarketing::API::SnippetManager->new();

    my $MockedSnippetManager = Test::MockObject::Extends->new($SnippetManager);

    TestExistsMethod($SnippetManager,'_createConversionTracker');
    $MockedSnippetManager->mock('_createConversionTracker', sub {
        $MockedSnippetManager->{'ConversionTracker'} = $self->_createMockedSnippet('ConversionTracker');
        return $MockedSnippetManager->{'ConversionTracker'};
    });
    TestExistsMethod($SnippetManager,'_createLeadTracker');
    $MockedSnippetManager->mock('_createLeadTracker', sub {
        $MockedSnippetManager->{'LeadTracker'} = $self->_createMockedSnippet('LeadTracker');
        return $MockedSnippetManager->{'LeadTracker'};
    });
    TestExistsMethod($SnippetManager,'_createRemarketingConfirmationPage');
    $MockedSnippetManager->mock('_createRemarketingConfirmationPage', sub {
        $MockedSnippetManager->{'RemarketingConfirmationPage'} = $self->_createMockedSnippetRemarketing('ConfirmationPage');
        return $MockedSnippetManager->{'RemarketingConfirmationPage'};
    });
    TestExistsMethod($SnippetManager,'_createRemarketingBasketPage');
    $MockedSnippetManager->mock('_createRemarketingBasketPage', sub {
        $MockedSnippetManager->{'RemarketingBasketPage'} = $self->_createMockedSnippetRemarketing('BasketPage');
        return $MockedSnippetManager->{'RemarketingBasketPage'};
    });
    TestExistsMethod($SnippetManager,'_createRemarketingCategoryPage');
    $MockedSnippetManager->mock('_createRemarketingCategoryPage', sub {
        $MockedSnippetManager->{'RemarketingCategoryPage'} = $self->_createMockedSnippetRemarketing('CategoryPage');
        return $MockedSnippetManager->{'RemarketingCategoryPage'};
    });
    TestExistsMethod($SnippetManager,'_createRemarketingProductPage');
    $MockedSnippetManager->mock('_createRemarketingProductPage', sub {
        $MockedSnippetManager->{'RemarketingProductPage'} = $self->_createMockedSnippetRemarketing('ProductPage');
        return $MockedSnippetManager->{'RemarketingProductPage'};
    });
    TestExistsMethod($SnippetManager,'_createRemarketingHomePage');
    $MockedSnippetManager->mock('_createRemarketingHomePage', sub {
        $MockedSnippetManager->{'RemarketingHomePage'} = $self->_createMockedSnippetRemarketing('HomePage');
        return $MockedSnippetManager->{'RemarketingHomePage'};
    });
    return $MockedSnippetManager;
}



sub _createMockedSnippet{
    my $self = shift;
    my ($ClassName) = @_;
    TestExistsMethod("DE_EPAGES::EasyMarketing::API::Snippet::$ClassName",'getSnippet');
    my $MockedSnippet = Test::MockObject::Extends->new("DE_EPAGES::EasyMarketing::API::Snippet::$ClassName", {});

    $MockedSnippet->mock('getSnippet', sub {
        shift;
        my ($Object) = @_;
        $MockedSnippet->{'getSnippet_Params'} = $Object;
        return "Snippet for $ClassName";
    });
    return $MockedSnippet;
}



sub _createMockedSnippetRemarketing{
    my $self = shift;
    my ($ClassName) = @_;
    TestExistsMethod("DE_EPAGES::EasyMarketing::API::Snippet::Remarketing::$ClassName",'getSnippet');
    my $MockedSnippet = Test::MockObject::Extends->new("DE_EPAGES::EasyMarketing::API::Snippet::Remarketing::$ClassName", {});

    $MockedSnippet->mock('getSnippet', sub {
        shift;
        my ($Object) = @_;
        $MockedSnippet->{'getSnippet_Params'} = $Object;
        return "Snippet for Remarketing $ClassName";
    });
    return $MockedSnippet;
}



sub _createClass {
    my $self = shift;
    my ($HasEasyMarketingAccessToken) = @_;
    my $MockifyObject = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::Object::API::Object::Class',
        []
    );
    $MockifyObject->addMockWithReturnValueAndParameterCheck('existsAttribute', $HasEasyMarketingAccessToken, [{'string' => 'EasyMarketingAccessToken'}]);
    return $MockifyObject->getMockObject();
}



__PACKAGE__->runScript();

1;